/*
 *  Copyright 2001 Adrian Thurston <adriant@ragel.ca>
 */

/*  This file is part of Ragel.
 *
 *  Ragel is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Ragel is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Ragel; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */


#include "fsm.h"

#include "fsmgraph.cc"
template class FsmTrans< State, Trans, int, OrdCmp<int> >;
template class FsmState<State, int, Trans>;
template class FsmGraph<State, int, Trans>;

void DumpTransition( ostream &out, int onChar, Trans *trans )
{
	out.form(" %.2i %.2x [%i](", trans->toState->num,
			onChar, trans->priority );

	/* Print the functions, adding in commas in between. */
//	Trans::TransFuncEl *tfel = trans->transFuncTable.table;
//	int ntfel = trans->transFuncTable.tableLength;
//	for ( int i = 0; i < ntfel-1; i++, tfel++ )
//		out.form("%s,", tfel->value);
//
//	/* Print the last function without the comma. */
//	if ( ntfel > 0 )
//		out.form("%s", tfel->value);
		
	/* Print the functions, adding in commas in between. */
	Trans::TransFuncEl *tfel = trans->transFuncTable.table;
	int nfunc = trans->transFuncTable.tableLength;
	for ( int i = 0; i < nfunc-1; i++, tfel++ )
		out.form("%i,", tfel->value);

	/* Print the last function without the comma. */
	if ( nfunc > 0 )
		out.form("%i", tfel->value);
		
	out << ") |";
}

void DumpTransitions( ostream &out, State::TransListType *list )
{
	State::TransEl *tel = list->table;
	int ntel = list->tableLength;
	for ( int i = 0; i < ntel; i++, tel++ )
		DumpTransition( out, tel->key, tel->value );

	out << endl;
}

void DumpGraph( ostream &out, Fsm *fsm )
{
	/* Set the state numbers so we print something nice! */
	fsm->SetStateNumbers();

	/* Some header info. */
	out.form("StartState: %.2i  ", fsm->StartState->num);
	out.form("Final States:");

	/* Dump Final states. */
	State **st = fsm->FinStateSet.table;
	int nst = fsm->FinStateSet.tableLength;
	for ( int i = 0; i < nst; i++, st++ )
		out.form(" %.2i |", (*st)->num );
	out << endl;

	/* Walk the list of states, printing. */
	State *state = fsm->head;
	while (state) {
		out.form("%.2i [%i] (", state->num, state->outPriority );

//		/* Print the functions, adding in commas in between. */
//		Trans::TransFuncEl *tfel = state->outTransition.transFuncTable.table;
//		int ntfel = state->outTransition.transFuncTable.tableLength;
//		for ( int i = 0; i < ntfel-1; i++, tfel++ )
//			out.form("%i,", tfel->value);
//
//		/* Print the last function without the comma. */
//		if ( ntfel > 0 )
//			out.form("%i", tfel->value);

		/* Print the functions, adding in commas in between. */
		Trans::TransFuncEl *tfel = state->outTransFuncTable.table;
		int nfunc = state->outTransFuncTable.tableLength;
		for ( int i = 0; i < nfunc-1; i++, tfel++ )
			out.form("%i,", tfel->value);

		/* Print the last function without the comma. */
		if ( nfunc > 0 )
			out.form("%i", tfel->value);

		out << ")\n->";

		DumpTransitions(out, &state->outList);
		state = state->next;
	}
}

